@@ -12,22 +12,16 @@
 #include <signal.h>
 #include <sys/wait.h>
 
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-authentication-manager.h>
 #include <libgnomeui/gnome-ui-init.h>
 
-#include <libbonoboui.h>
-#include <bonobo-activation/bonobo-activation.h>
-
-#include "menu.h"
-#include "panel.h"
-#include "panel-util.h"
 #include "panel-gconf.h"
 #include "panel-profile.h"
 #include "panel-config-global.h"
 #include "panel-shell.h"
 #include "panel-multiscreen.h"
 #include "panel-session.h"
-#include "xstuff.h"
 #include "panel-stock-icons.h"
 #include "panel-action-protocol.h"
 #include "panel-lockdown.h"
@@ -40,37 +34,47 @@ GSList *panel_list = NULL;
 
 GtkTooltips *panel_tooltips = NULL;
 
-static const struct poptOption options[] = {
-  {NULL, '\0', 0, NULL, 0}
+static const GOptionEntry options[] = {
+  { NULL }
 };
 
 int
 main (int argc, char **argv)
 {
+	GOptionContext *context;
+
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	context = g_option_context_new ("");
+
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
 	gnome_program_init ("gnome-panel", VERSION,
 			    LIBGNOMEUI_MODULE,
 			    argc, argv,
-			    GNOME_PARAM_POPT_TABLE, options,
+			    GNOME_PARAM_GOPTION_CONTEXT, context,
 			    GNOME_PROGRAM_STANDARD_PROPERTIES,
 			    NULL);
 
+	g_option_context_free (context);
+
+	gtk_window_set_default_icon_name ("gnome-panel");
+
 	if (!panel_shell_register ())
 		return -1;
 
 	gnome_authentication_manager_init ();
 
-	gtk_window_set_default_icon_name ("gnome-panel");
-
 	panel_tooltips = gtk_tooltips_new ();
 
 	panel_action_protocol_init ();
 	panel_multiscreen_init ();
 	panel_init_stock_icons_and_items ();
 
+	panel_session_init ();
+
 	gconf_client_add_dir (panel_gconf_get_client (),
 			      "/desktop/gnome/interface",
 			      GCONF_CLIENT_PRELOAD_NONE,
@@ -80,8 +84,6 @@ main (int argc, char **argv)
 	panel_lockdown_init ();
 	panel_profile_load ();
 
-	panel_session_init (argv [0]);
-
 	/*add forbidden lists to ALL panels*/
 	g_slist_foreach (panels,
 			 (GFunc)panel_widget_add_forbidden,
@@ -91,5 +93,9 @@ main (int argc, char **argv)
 
 	panel_lockdown_finalize ();
 
+	gconf_client_remove_dir (panel_gconf_get_client (),
+				 "/desktop/gnome/interface",
+				 NULL);
+
 	return 0;
 }
