@@ -260,8 +260,8 @@ reload_image_menu_items (void)
 }
 
 static void
-icon_theme_changed (GnomeIconTheme *icon_theme,
-		    gpointer        data)
+icon_theme_changed (GtkIconTheme *icon_theme,
+		    gpointer      data)
 {
 	reload_image_menu_items ();
 }
@@ -785,21 +785,33 @@ add_menu_to_panel (GtkWidget      *menuitem,
 		   GMenuTreeEntry *entry)
 {
 	GMenuTreeDirectory *directory;
+	GMenuTree          *tree;
 	PanelWidget        *panel;
 	PanelData          *pd;
 	int                 insertion_pos;
 	char               *menu_path;
-	char               *menu_filename;
+	const char         *menu_filename;
 
-	menu_filename = g_object_get_data (G_OBJECT (menuitem->parent),
-					   "panel-menu-tree-filename");
-
-	if (!menu_filename) {
-		g_warning ("Cannot find the filename for the menu");
+	directory = gmenu_tree_item_get_parent (GMENU_TREE_ITEM (entry));
+	if (!directory) {
+		g_warning ("Cannot find the filename for the menu: no directory");
 		return;
 	}
 
-	directory = gmenu_tree_item_get_parent (GMENU_TREE_ITEM (entry));
+	tree = gmenu_tree_directory_get_tree (directory);
+	if (!tree) {
+		gmenu_tree_item_unref (directory);
+		g_warning ("Cannot find the filename for the menu: no tree");
+		return;
+	}
+
+	menu_filename = gmenu_tree_get_menu_file (tree);
+	gmenu_tree_unref (tree);
+	if (!menu_filename) {
+		gmenu_tree_item_unref (directory);
+		g_warning ("Cannot find the filename for the menu: no filename");
+		return;
+	}
 
 	panel = menu_get_panel (menuitem);
 
@@ -876,13 +888,14 @@ static GtkWidget *
 create_item_context_menu (GtkWidget   *item,
 			  PanelWidget *panel_widget)
 {
-	GMenuTreeEntry *entry;
-	GtkWidget      *parent;
-	GtkWidget      *menu;
-	GtkWidget      *submenu;
-	GtkWidget      *menuitem;
-	char           *menu_filename;
-	gboolean        id_lists_writable;
+	GMenuTreeEntry     *entry;
+	GMenuTreeDirectory *directory;
+	GMenuTree          *tree;
+	GtkWidget          *menu;
+	GtkWidget          *submenu;
+	GtkWidget          *menuitem;
+	const char         *menu_filename;
+	gboolean            id_lists_writable;
 
 	id_lists_writable = panel_profile_id_lists_are_writable ();
 
@@ -890,19 +903,19 @@ create_item_context_menu (GtkWidget   *item,
 	if (!entry)
 		return NULL;
 
-	parent = item->parent;
+	directory = gmenu_tree_item_get_parent (GMENU_TREE_ITEM (entry));
+	if (!directory)
+		return NULL;
 
-	menu_filename = NULL;
-	while (parent) {
-		menu_filename = g_object_get_data (G_OBJECT (parent),
-						   "panel-menu-tree-filename");
-		if (menu_filename)
-			break;
+	tree = gmenu_tree_directory_get_tree (directory);
+	gmenu_tree_item_unref (directory);
+	if (!tree)
+		return NULL;
 
-		parent = gtk_menu_get_attach_widget (GTK_MENU (parent))->parent;
-		if (!GTK_IS_MENU (parent))
-			break;
-	}
+	menu_filename = gmenu_tree_get_menu_file (tree);
+	gmenu_tree_unref (tree);
+	if (!menu_filename)
+		return NULL;
 
 	menu = create_empty_menu ();
 	g_object_set_data_full (G_OBJECT (item),
@@ -919,7 +932,7 @@ create_item_context_menu (GtkWidget   *item,
 	setup_menuitem (menuitem,
 			panel_menu_icon_get_size (),
 			NULL,
-			_("Add this launcher to panel"),
+			_("Add this launcher to _panel"),
 			FALSE);
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (add_app_to_panel), entry);
@@ -930,7 +943,7 @@ create_item_context_menu (GtkWidget   *item,
 	setup_menuitem (menuitem,
 			panel_menu_icon_get_size (),
 			NULL,
-			_("Add this launcher to desktop"),
+			_("Add this launcher to _desktop"),
 			FALSE);
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (add_app_to_desktop), entry);
@@ -941,16 +954,12 @@ create_item_context_menu (GtkWidget   *item,
 	submenu = create_empty_menu ();
 
 	g_object_set_data (G_OBJECT (submenu), "menu_panel", panel_widget);
-	g_object_set_data_full (G_OBJECT (submenu),
-				"panel-menu-tree-filename",
-				g_strdup (menu_filename),
-				(GDestroyNotify) g_free);
 
 	menuitem = gtk_image_menu_item_new ();
 	setup_menuitem (menuitem,
 			panel_menu_icon_get_size (),
 			NULL,
-			_("Entire menu"),
+			_("_Entire menu"),
 			FALSE);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), submenu);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
@@ -960,7 +969,7 @@ create_item_context_menu (GtkWidget   *item,
 	setup_menuitem (menuitem,
 			panel_menu_icon_get_size (),
 			NULL,
-			_("Add this as drawer to panel"),
+			_("Add this as _drawer to panel"),
 			FALSE);
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (add_menudrawer_to_panel), entry);
@@ -972,7 +981,7 @@ create_item_context_menu (GtkWidget   *item,
 	setup_menuitem (menuitem,
 			panel_menu_icon_get_size (),
 			NULL,
-			_("Add this as menu to panel"),
+			_("Add this as _menu to panel"),
 			FALSE);
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (add_menu_to_panel), entry);
@@ -1144,63 +1153,34 @@ image_menuitem_size_request (GtkWidget      *menuitem,
 	requisition->height = MAX (requisition->height, icon_height);
 }
 
-static void
-setup_mnemonic (GtkWidget *menuitem,
-		GtkWidget *old_toplevel,
-		gpointer   keyval_as_pointer)
-{
-	GtkWidget *toplevel;
-	guint      keyval;
-
-	keyval = GPOINTER_TO_UINT (keyval_as_pointer);
-
-	if (old_toplevel != NULL)
-		gtk_window_remove_mnemonic (GTK_WINDOW (old_toplevel),
-					    keyval,
-					    GTK_WIDGET (menuitem));
-
-	toplevel = gtk_widget_get_toplevel (menuitem);
-	if (GTK_WIDGET_TOPLEVEL (toplevel))
-		gtk_window_add_mnemonic (GTK_WINDOW (toplevel),
-					 keyval,
-					 GTK_WIDGET (menuitem));
-}
-
-static void
-setup_invisible_mnemonic (GtkWidget *menuitem,
-			  char       unicode_char)
-{
-	guint accel_key;
-
-	accel_key = gdk_keyval_to_lower (gdk_unicode_to_keyval (unicode_char));
-
-	g_signal_connect (menuitem, "hierarchy_changed", 
-			  G_CALLBACK (setup_mnemonic),
-			  GUINT_TO_POINTER (accel_key));
-
-	setup_mnemonic (menuitem, NULL, GUINT_TO_POINTER (accel_key));
-}
-
 void
 setup_menuitem (GtkWidget   *menuitem,
 		GtkIconSize  icon_size,
 		GtkWidget   *image,
 		const char  *title,
-		gboolean     invisible_mnemonic)
+		gboolean     create_invisible_mnemonic)
 			       
 {
 	GtkWidget *label;
 
-	if (invisible_mnemonic)
-		label = gtk_label_new (title);
-	else
+	if (create_invisible_mnemonic) {
+		gchar *_title;
+
+		label = g_object_new (GTK_TYPE_ACCEL_LABEL, NULL);
+		_title = g_strconcat ("_", title, NULL);
+		gtk_label_set_text_with_mnemonic (GTK_LABEL (label), _title);
+		g_free (_title);
+
+		gtk_label_set_pattern (GTK_LABEL (label), "");
+
+		gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (label),
+						  menuitem);
+	} else
 		label = gtk_label_new_with_mnemonic (title);
+
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_widget_show (label);
        
-	if (invisible_mnemonic && title)
-		setup_invisible_mnemonic (menuitem, title [0]);
-	
 	gtk_container_add (GTK_CONTAINER (menuitem), label);
 
 	if (image) {
@@ -1247,7 +1227,8 @@ setup_uri_drag (GtkWidget  *menuitem,
 			     menu_item_targets, 1,
 			     GDK_ACTION_COPY);
 
-	gtk_drag_source_set_icon_name (menuitem, icon);
+	if (icon != NULL)
+		gtk_drag_source_set_icon_name (menuitem, icon);
 	
 	g_signal_connect (G_OBJECT (menuitem), "drag_begin",
 			  G_CALLBACK (drag_begin_menu_cb), NULL);
@@ -1303,14 +1284,23 @@ submenu_to_display (GtkWidget *menu)
 
 	g_object_set_data (G_OBJECT (menu), "panel-menu-needs-loading", NULL);
 
-	if (!(directory = g_object_get_data (G_OBJECT (menu), "panel-menu-tree-directory")) &&
-	     (menu_path = g_object_get_data (G_OBJECT (menu), "panel-menu-tree-path"))) {
-		if (!(tree = g_object_get_data (G_OBJECT (menu), "panel-menu-tree")))
+	directory = g_object_get_data (G_OBJECT (menu),
+				       "panel-menu-tree-directory");
+	if (!directory) {
+		menu_path = g_object_get_data (G_OBJECT (menu),
+					       "panel-menu-tree-path");
+		if (!menu_path)
 			return;
 
-		directory = gmenu_tree_get_directory_from_path (tree, menu_path);
+		tree = g_object_get_data (G_OBJECT (menu), "panel-menu-tree");
+		if (!tree)
+			return;
 
-		g_object_set_data_full (G_OBJECT (menu), "panel-menu-tree-directory",
+		directory = gmenu_tree_get_directory_from_path (tree,
+								menu_path);
+
+		g_object_set_data_full (G_OBJECT (menu),
+					"panel-menu-tree-directory",
 					directory,
 					(GDestroyNotify) gmenu_tree_item_unref);
 	}
@@ -1462,6 +1452,8 @@ create_menuitem (GtkWidget          *menu,
 				(GDestroyNotify) gmenu_tree_item_unref);
 
 	if (alias_directory)
+		//FIXME: we should probably use this data when we do dnd or
+		//context menu for this menu item
 		g_object_set_data_full (G_OBJECT (menuitem),
 					"panel-menu-tree-alias-directory",
 					gmenu_tree_item_ref (alias_directory),
@@ -1481,7 +1473,10 @@ create_menuitem (GtkWidget          *menu,
 					  gmenu_tree_entry_get_name (entry),
 			TRUE);
 
-	if (gmenu_tree_entry_get_comment (entry))
+	if ((alias_directory &&
+	     gmenu_tree_directory_get_comment (alias_directory)) ||
+	    (!alias_directory &&
+	     gmenu_tree_entry_get_comment (entry)))
 		gtk_tooltips_set_tip (panel_tooltips,
 				      menuitem,
 				      alias_directory ? gmenu_tree_directory_get_comment (alias_directory) :
@@ -1501,8 +1496,10 @@ create_menuitem (GtkWidget          *menu,
 				     menu_item_targets, 1,
 				     GDK_ACTION_COPY);
 
-		gtk_drag_source_set_icon_name (menuitem,
-					       gmenu_tree_entry_get_icon (entry));
+		//FIXME should remove .png at the end of the icon name
+		if (gmenu_tree_entry_get_icon (entry) != NULL)
+			gtk_drag_source_set_icon_name (menuitem,
+						       gmenu_tree_entry_get_icon (entry));
 		g_signal_connect (G_OBJECT (menuitem), "drag_begin",
 				  G_CALLBACK (drag_begin_menu_cb), NULL);
 		g_signal_connect (menuitem, "drag_data_get",
@@ -1549,6 +1546,8 @@ static void
 handle_gmenu_tree_changed (GMenuTree *tree,
 			   GtkWidget *menu)
 {
+	guint idle_id;
+
 	while (GTK_MENU_SHELL (menu)->children)
                 gtk_widget_destroy (GTK_MENU_SHELL (menu)->children->data);
 
@@ -1563,6 +1562,15 @@ handle_gmenu_tree_changed (GMenuTree *tree,
 	g_object_set_data (G_OBJECT (menu),
 			   "panel-menu-needs-appending",
 			   GUINT_TO_POINTER (TRUE));
+
+	idle_id = g_idle_add_full (G_PRIORITY_LOW,
+				   submenu_to_display_in_idle,
+				   menu,
+				   NULL);
+	g_object_set_data_full (G_OBJECT (menu),
+				"panel-menu-idle-id",
+				GUINT_TO_POINTER (idle_id),
+				remove_submenu_to_display_idle);
 }
 
 static void
@@ -1586,11 +1594,6 @@ create_applications_menu (const char *menu_file,
 
 	tree = gmenu_tree_lookup (menu_file, GMENU_TREE_FLAGS_NONE);
 
-	g_object_set_data_full (G_OBJECT (menu),
-				"panel-menu-tree-filename",
-				g_strdup (menu_file),
-				(GDestroyNotify) g_free);
-
 	g_object_set_data_full (G_OBJECT (menu),
 				"panel-menu-tree",
 				gmenu_tree_ref (tree),
@@ -1695,13 +1698,13 @@ setup_menu_item_with_icon (GtkWidget   *item,
 			   const char  *icon_name,
 			   const char  *stock_id,
 			   const char  *title,
-			   gboolean     invisible_mnemonic)
+			   gboolean     create_invisible_mnemonic)
 {
 	if (icon_name || stock_id)
 		panel_load_menu_image_deferred (item, icon_size,
 						stock_id, icon_name, NULL);
 
-	setup_menuitem (item, icon_size, NULL, title, invisible_mnemonic);
+	setup_menuitem (item, icon_size, NULL, title, create_invisible_mnemonic);
 }
 
 static void
